/** CLI.java defines a command-line interface for the cliser system.
 *  @author  Joel Adams, Spring 2000.
 *  All rights reserved
 */

package cliser;

import cliser.*;                 // Generator, JavaGenerator, ...
import java.io.*;                // BufferedReader

class CLI extends Object
{
    /** command-line interface for Cliser
     *  Input: serviceName, port, network interface, protocol,
     *          gen-client, gen-server,
     *          multithreaded (if gen-server).
     *  Output: Client source code (if gen-client);
     *          Server source coe (if gen-server).
     */
    public static void main(String [] args)
    {
      try
      {
	System.out.println("\nWelcome to Cliser, the TCP/IP client-server "
			   + "code-generation system!");

	// Get name of service
	System.out.print("\nWhat is the name of your service (e.g., ftp)? ");
        String usersService = keyboard.readLine();

	int usersPort = getPort();

	String usersProtocol = getProtocol();

	String usersSourceLanguage = getSourceLanguage();

	
	Generator generator = null;

	if ( usersSourceLanguage.equals("Java") )
	    generator = new JavaGenerator(usersService, usersPort, usersProtocol);
	else if ( usersSourceLanguage.equals("C++") )
        {
	    String usersNetInterface = getNetworkInterface();
	    generator = new CPlusPlusGenerator(usersService, usersPort,
					       usersProtocol, usersNetInterface);
	}
	else
         {
	    System.err.println("CLI.main(): "
			       + " unexpected source language: "
			       + usersSourceLanguage );
	    System.exit(1);
	 }

	boolean buildClient = getBuildClient();
	if (buildClient)
	    generator.generateClient();

	final boolean MULTITHREADED = true,
	              SINGLETHREADED = false;
	boolean buildServer = getBuildServer();
	if (buildServer)
        {
	    boolean concurrent = getConcurrent();
	    if (concurrent)
		generator.generateServer(MULTITHREADED);
	    else
		generator.generateServer(SINGLETHREADED);
	}
      }
      catch (IOException e) {System.err.println(e); System.exit(1);}
    }


    private static int getPort()
    {
	int result = -1;

    portLoop:
	for (;;)
        {
	    System.out.print("\nWhat port should it use? ");
	    try 
             { result = Integer.parseInt( keyboard.readLine() ); }
	    catch (Exception e) 
             { System.err.println("\n*** Port must be a positive integer.\n"); 
	       continue portLoop; }

	    if (result > 0) break;
	    System.err.println("\n*** Port must be a postive integer.\n");
	}

	return result;
    }

    private static String getNetworkInterface()
    {
	String result = "";
	boolean choiceInvalid = true;
    netLoop:
	do
	{
	    System.out.print("\nWhat network interface should it use:\n"
			     + "\ta - BSD sockets (BSD Unix, Linux, ...)\n"
			     + "\tb - Winsock sockets (MS-Windows)\n"
			     + "\tc - TLI (System V Unix, Solaris, ...)\n"
			     + "--> ");

	    char interfaceChoice = '\0';
	    try 
		{ interfaceChoice = keyboard.readLine().charAt(0); }
	    catch (IOException e)
		{ System.err.println(e);
		  continue netLoop; 
		}
		    
	    switch (interfaceChoice)
		{
		case 'a': case 'A': 
		    result = "BSD"; 
		    choiceInvalid = false;
		    break;
		case 'b': case 'B': 
		    result = "Winsock"; 
		    choiceInvalid = false;
		    break;
		case 'c': case 'C': 
		    result = "TLI";
		    choiceInvalid = false;
		    break;
		default:
		    System.err.println("\n*** " + interfaceChoice
				       + " is not a valid menu choice!\n");
		    choiceInvalid = true;
		}
	}
	while (choiceInvalid);

	return result;
    }


    private static String getProtocol()
    {
	String result = "";
	boolean choiceInvalid = true;
    protocolLoop:
	do
	{
	    System.out.print("\nWhat network protocol do you want to use:\n"
			       + "\ta - TCP (reliable but slow)\n"
			       + "\tb - UDP (fast but unreliable)\n"
			       + "--> ");
	    char protocolChoice = '\0';
	    try { protocolChoice = keyboard.readLine().charAt(0); }
	    catch (IOException e)
		{ System.err.println(e);
		continue protocolLoop;
		}
	    switch (protocolChoice)
		{
		case 'a': case 'A': 
		    result = "TCP"; 
		    choiceInvalid = false;
		    break;
		case 'b': case 'B': 
		    result = "UDP"; 
		    choiceInvalid = false;
		    break;
		default:
		    System.err.println("\n*** " + protocolChoice
				       + " is not a valid menu choice!\n");
		    choiceInvalid = true;
		}
	}
	while (choiceInvalid);

	return result;
    }


    private static boolean getConcurrent()
    {
	boolean result = false;
	boolean threadInvalid = true;
    concurrentLoop:
	do
        {
	    System.out.print("\nShould the server be:\n"
			     + "\ta - Iterative (singlethreaded)\n"
			     + "\tb - Concurrent (multithreaded)\n"
			     + "--> ");
	    char threadChoice = '\0';
	    try { threadChoice = keyboard.readLine().charAt(0); }
	    catch (IOException e)
		{ System.err.println(e);
		continue concurrentLoop;
		}
	    switch (threadChoice)
            {
	    case 'a': case 'A':
		result = false;
		threadInvalid = false;
		break;
	    case 'b': case 'B':
		result = true;
		threadInvalid = false;
		break;
	    default:
		System.err.println("\n** " + threadChoice
				   + " is invalid\n");
		threadInvalid = true;
	    }
	}
	while (threadInvalid);

	return result;
    }

    private static boolean getBuildServer()
    {
	boolean result = false;
	boolean choiceInvalid = true;
    buildLoop:
	do
	{
	    System.out.print("\nDo you want to generate a server (y or n)? ");
	    char serverChoice = '\0';
	    try { serverChoice = keyboard.readLine().charAt(0); }
	    catch (IOException e)
		{ System.err.println(e);
		continue buildLoop;
		}

	    switch (serverChoice)
		{
		case 'y': case 'Y':
		    result = true;
		    choiceInvalid = false;
		    break;
		case 'n': case 'N':
		    result = false;
		    choiceInvalid = false;
		default:
		    choiceInvalid = true;
		}
	}
	while (choiceInvalid);

	return result;
    }

    private static boolean getBuildClient()
    {
	boolean result = false;
	boolean choiceInvalid = true;
    buildLoop:
	do
	{
	    System.out.print("\nDo you want to generate a client (y or n)? ");
	    char clientChoice = '\0';
	    try { clientChoice = keyboard.readLine().charAt(0); }
	    catch (IOException e)
		{ System.err.println(e);
		continue buildLoop;
		}
	    switch (clientChoice)
		{
		case 'y': case 'Y':
		    result = true;
		    choiceInvalid = false;
		    break;
		case 'n': case 'N':
		    result = false;
		    choiceInvalid = false;
		    break;
		default:
		    choiceInvalid = true;
		}
	}
	while (choiceInvalid);

	return result;
    }

    private static String getSourceLanguage()
    {
	String result = null;
	boolean choiceInvalid = true;
    sourceLoop:
	do
        {
	    System.out.print("\nGenerate souce code in what "
			     + "programming language:\n"
			     + "\ta - C++\n"
			     + "\tb - Java\n"
			     + "--> ");
	    char languageChoice = '\0';
	    try { languageChoice = keyboard.readLine().charAt(0); }
	    catch (IOException e)
		{ System.err.println(e);
		continue sourceLoop;
		}
	    switch (languageChoice)
            {
	    case 'a': case 'A':
		result = "C++";
		choiceInvalid = false;
		break;
	    case 'b': case 'B':
		result = "Java";
		choiceInvalid = false;
	    default:
		choiceInvalid = true;
	    }
	}
	while (choiceInvalid);

	return result;
    }


    private static BufferedReader keyboard = new BufferedReader(
			                      new InputStreamReader(System.in) );
}

